//+--------------------------------------------------------------------------------------------+
//|                                                                              FindHoles.mq4 |
//|                                                             Copyright  2012, Serega Lykov |
//|                                                                 http://mtexperts.narod.ru/ |
//+--------------------------------------------------------------------------------------------+

#property copyright "Copyright  2012, Serega Lykov"
#property link      "http://mtexperts.narod.ru/"

#property show_inputs

//---- constants ------------------------------------------------------------------------------+
#define ScriptName               "FindHoles"
#define SecondsInHoleOverWeek    216000

//---- external parameters --------------------------------------------------------------------+
extern int      Run_Method                      = 0;
extern int      SecondsInHole                   = 600;
extern datetime FromDate                        = D'2000.01.01 00:00:00';
extern string   DataTXT_FileName                = "";
extern int      DataTXT_GMTOffset               = 2;

//---- global variables -----------------------------------------------------------------------+
static string   symbol;
static string   str_comment;
static string   str_header = "Script ";
static string   str_err_01 = "Error: can not open file\"";
static string   str_mes_01 = "Can not find bars.";
static string   str_mes_02 = "Can not find holes.";
static string   str_mes_03 = "error\n";
static string   str_mes_04 = "process...\n";
static string   str_mes_05 = "writed ";
static string   str_mes_06 = " bars";
static string   str_mes_07 = "Ok\n";

//---------------------------------------------------------------------------------------------+
//---- init -----------------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int init()
  {
   str_comment = StringConcatenate(str_header,ScriptName,":\n");
   Comment(str_comment);
   if(Run_Method > 6 || Run_Method < 0) Run_Method = 0;
   symbol = Symbol();
   return(0);
   Print("---- Programming by Serega Lykov, http://mtexperts.narod.ru/ ----");
  }

//---------------------------------------------------------------------------------------------+
//---- deinit ---------------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int deinit()
  {
   Comment("");
   return(0);
  }

//---------------------------------------------------------------------------------------------+
//---- start ----------------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int start()
  {
   switch(Run_Method)
     {
      case 1:
         FindHoles();
         break;
      case 2:
         FindData();
         break;
      case 3:
         ImportData();
         break;
      case 4:
         PeriodConverterEx();
         break;
      case 0:
         FindHoles();
      case 5:
         FindData();
      case 6:
         ImportData();
         PeriodConverterEx();
         break;
     }
   return(0);
  }

//---------------------------------------------------------------------------------------------+
//---- FindHoles ------------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
void FindHoles()
  {
   Comment(StringConcatenate(str_comment,"FindHoles - ",str_mes_04));
   string file_name = StringConcatenate(ScriptName,"_",StringSubstr(symbol,0,6),".txt");
   int file_handle_txt = FileOpen(file_name,FILE_WRITE|FILE_CSV);
   if(file_handle_txt < 0)
     {
      Print(str_err_01,file_name,"\".");
      str_comment = StringConcatenate(str_comment,"FindHoles - ",str_mes_03);
      Comment(str_comment);
      return;
     }
   file_name = StringConcatenate(ScriptName,"_",StringSubstr(symbol,0,6),".csv");
   int file_handle_csv = FileOpen(file_name,FILE_WRITE|FILE_CSV);
   if(file_handle_csv < 0)
     {
      Print(str_err_01,file_name,"\".");
      FileClose(file_handle_txt);
      str_comment = StringConcatenate(str_comment,"FindHoles - ",str_mes_03);
      Comment(str_comment);
      return;
     }
   int bars = iBars(NULL,PERIOD_M1);
   for(int k=bars-1; k>=0; k--)
     {
      if(iTime(NULL,PERIOD_M1,k) >= FromDate)
        {
         if(k < (bars - 1)) k++;
         break;
        }
     }
   if(k <= 0)
     {
      FileWrite(file_handle_txt,str_mes_01);
      FileClose(file_handle_txt);
      FileClose(file_handle_csv);
      str_comment = StringConcatenate(str_comment,"FindHoles - ",str_mes_07);
      Comment(str_comment);
      return;
     }
   datetime prev_bartime = iTime(NULL,PERIOD_M1,k);
   int prev_day_of_week  = TimeDayOfWeek(prev_bartime);
   int num_holes = 0;
   for(int i=k-1; i>=0; i--)
     {
      datetime curr_bartime = iTime(NULL,PERIOD_M1,i);
      int curr_day_of_week  = TimeDayOfWeek(curr_bartime);
      int time_between_bars = curr_bartime - prev_bartime;
      if(time_between_bars >= SecondsInHole)
        {
         if(time_between_bars >= SecondsInHoleOverWeek || curr_day_of_week >= prev_day_of_week)
           {
            num_holes++;
            string str_hole = "";
            int minutes_between_bars = time_between_bars / 60;
            if(minutes_between_bars >= 60)
              {
               int hours_between_bars = minutes_between_bars / 60;
               if(hours_between_bars >= 24)
                 {
                  int days_between_bars = hours_between_bars / 24;
                  hours_between_bars = hours_between_bars % 24;
                  if(days_between_bars == 1) string str_days = " day ";
                  else str_days = " days ";
                  str_hole = StringConcatenate(DoubleToStr(days_between_bars,0),str_days);
                 }
               str_hole = StringConcatenate(str_hole,IntegerToStr(hours_between_bars,2),":");
               minutes_between_bars = minutes_between_bars % 60;
              }
            time_between_bars = time_between_bars % 60;
            string str_time_between_bars = StringConcatenate(str_hole,IntegerToStr(minutes_between_bars,2),":",IntegerToStr(time_between_bars,2));
            FileWrite(file_handle_txt,StringConcatenate("#",IntegerToStr(num_holes,4),"  ",TimeToStr(prev_bartime,TIME_DATE|TIME_MINUTES|TIME_SECONDS),"(",DayOfWeekToStr(prev_day_of_week,true),") --- ",TimeToStr(curr_bartime,TIME_DATE|TIME_MINUTES|TIME_SECONDS),"(",DayOfWeekToStr(curr_day_of_week,true),")  ",str_time_between_bars));
            FileWrite(file_handle_csv,StringConcatenate("#",IntegerToStr(num_holes,4)),TimeToStr(prev_bartime,TIME_DATE|TIME_MINUTES|TIME_SECONDS),TimeToStr(curr_bartime,TIME_DATE|TIME_MINUTES|TIME_SECONDS));
           }
        }
      prev_bartime     = curr_bartime;
      prev_day_of_week = curr_day_of_week;
     }
   if(num_holes == 0) FileWrite(file_handle_txt,str_mes_02);
   FileClose(file_handle_txt);
   FileClose(file_handle_csv);
   str_comment = StringConcatenate(str_comment,"FindHoles - ",str_mes_07);
   Comment(str_comment);
  }

//---------------------------------------------------------------------------------------------+
//---- FindData -------------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
void FindData()
  {
   Comment(StringConcatenate(str_comment,"FindData - ",str_mes_04));
   int gmt_offset_seconds = DataTXT_GMTOffset * 3600;
   if(StringLen(DataTXT_FileName) <= 0) DataTXT_FileName = StringConcatenate(StringSubstr(symbol,0,6),".txt");
   int file_handle_txt = FileOpen(DataTXT_FileName,FILE_READ|FILE_CSV,",");
   if(file_handle_txt < 0)
     {
      Print(str_err_01,DataTXT_FileName,"\".");
      str_comment = StringConcatenate(str_comment,"FindData - ",str_mes_03);
      Comment(str_comment);
      return;
     }
   string file_name = StringConcatenate(ScriptName,"_",StringSubstr(symbol,0,6),".csv");
   int file_handle_csv = FileOpen(file_name,FILE_READ|FILE_CSV);
   if(file_handle_csv < 0)
     {
      Print(str_err_01,file_name,"\".");
      FileClose(file_handle_txt);
      str_comment = StringConcatenate(str_comment,"FindData - ",str_mes_03);
      Comment(str_comment);
      return;
     }
   file_name = StringConcatenate(ScriptName,"_FindData_",StringSubstr(symbol,0,6),".csv");
   int file_handle_result = FileOpen(file_name,FILE_WRITE|FILE_READ|FILE_CSV);
   if(file_handle_result < 0)
     {
      Print(str_err_01,file_name,"\".");
      FileClose(file_handle_txt);
      FileClose(file_handle_csv);
      str_comment = StringConcatenate(str_comment,"FindData - ",str_mes_03);
      Comment(str_comment);
      return;
     }
   while(true)
     {
      if(FileIsEnding(file_handle_csv)) break;
      string csv_index = FileReadString(file_handle_csv);
      if(FileIsLineEnding(file_handle_csv)) continue;
      datetime hole_start_time = StrToTime(FileReadString(file_handle_csv));
      if(FileIsLineEnding(file_handle_csv)) continue;
      datetime hole_end_time   = StrToTime(FileReadString(file_handle_csv));
      bool txt_file_is_ending = false;
      while(true)
        {
         if(FileIsEnding(file_handle_txt))
           {
            txt_file_is_ending = true;
            break;
           }
         string txt_symbol = FileReadString(file_handle_txt);
         if(FileIsLineEnding(file_handle_txt)) continue;
         string txt_date = FileReadString(file_handle_txt);
         if(FileIsLineEnding(file_handle_txt)) continue;
         string txt_time = FileReadString(file_handle_txt);
         if(FileIsLineEnding(file_handle_txt)) continue;
         double txt_open  = StrToDouble(FileReadString(file_handle_txt));
         if(FileIsLineEnding(file_handle_txt)) continue;
         double txt_high  = StrToDouble(FileReadString(file_handle_txt));
         if(FileIsLineEnding(file_handle_txt)) continue;
         double txt_low   = StrToDouble(FileReadString(file_handle_txt));
         if(FileIsLineEnding(file_handle_txt)) continue;
         double txt_close = StrToDouble(FileReadString(file_handle_txt));
         if(FileIsLineEnding(file_handle_txt)) continue;
         int txt_volume = StrToInteger(FileReadString(file_handle_txt));
         datetime txt_bartime = StrToTime(StringConcatenate(StringSubstr(txt_date,6,2),".",StringSubstr(txt_date,4,2),".",StringSubstr(txt_date,0,4)," ",StringSubstr(txt_time,0,2),":",StringSubstr(txt_time,2,2),":",StringSubstr(txt_time,4,2)));
         txt_bartime += gmt_offset_seconds;
         if(txt_bartime <= hole_start_time) continue;
         if(txt_bartime >= hole_end_time) break;
         FileWrite(file_handle_result,TimeToStr(txt_bartime,TIME_DATE),TimeToStr(txt_bartime,TIME_MINUTES),DoubleToStr(txt_open,Digits),DoubleToStr(txt_high,Digits),DoubleToStr(txt_low,Digits),DoubleToStr(txt_close,Digits),DoubleToStr(txt_volume,0));
        }
      if(txt_file_is_ending) break;
     }
   FileClose(file_handle_txt);
   FileClose(file_handle_csv);
   FileClose(file_handle_result);
   str_comment = StringConcatenate(str_comment,"FindData - ",str_mes_07);
   Comment(str_comment);
  }

//---------------------------------------------------------------------------------------------+
//---- ImportData -----------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
void ImportData()
  {
   Comment(StringConcatenate(str_comment,"ImportData - ",str_mes_04));
   string file_name = StringConcatenate(ScriptName,"_old_",symbol,"1.hst");
   int file_handle_backup = FileOpenHistory(file_name,FILE_BIN|FILE_WRITE);
   if(file_handle_backup < 0)
     {
      Print(str_err_01,file_name,"\".");
      str_comment = StringConcatenate(str_comment,"ImportData - ",str_mes_03);
      Comment(str_comment);
	   return;
     }
   file_name = StringConcatenate(symbol,"1.hst");
   int file_handle_old_hst = FileOpenHistory(file_name,FILE_BIN|FILE_READ);
   if(file_handle_old_hst < 0)
     {
      Print(str_err_01,file_name,"\" (backup).");
      FileClose(file_handle_backup);
      str_comment = StringConcatenate(str_comment,"ImportData - ",str_mes_03);
      Comment(str_comment);
	   return;
     }
 	int hst_version = FileReadInteger(file_handle_old_hst);
 	FileWriteInteger(file_handle_backup,hst_version);
 	string hst_copyright = FileReadString(file_handle_old_hst,64);
  	FileWriteString (file_handle_backup,hst_copyright,64);
  	string hst_symbol = FileReadString(file_handle_old_hst,12);
   FileWriteString (file_handle_backup,hst_symbol,12);
   int hst_period = FileReadInteger(file_handle_old_hst);
  	FileWriteInteger(file_handle_backup,hst_period);
  	int hst_digits = FileReadInteger(file_handle_old_hst);
   FileWriteInteger(file_handle_backup,hst_digits);
   int hst_timesign = FileReadInteger(file_handle_old_hst);
  	FileWriteInteger(file_handle_backup,hst_timesign);
  	int hst_last_sync = FileReadInteger(file_handle_old_hst);
   FileWriteInteger(file_handle_backup,hst_last_sync);
   for(int i=0; i<13; i++)
     {
      int hst_reserved = FileReadInteger(file_handle_old_hst);
      FileWriteInteger(file_handle_backup,hst_reserved);
     }
   while(true)
     {
      if(FileIsEnding(file_handle_old_hst)) break;
      datetime old_hst_bartime = FileReadInteger(file_handle_old_hst);
      if(FileIsEnding(file_handle_old_hst)) break;
      double   old_hst_open    = FileReadDouble (file_handle_old_hst);
      if(FileIsEnding(file_handle_old_hst)) break;
      double   old_hst_low     = FileReadDouble (file_handle_old_hst);
      if(FileIsEnding(file_handle_old_hst)) break;
      double   old_hst_high    = FileReadDouble (file_handle_old_hst);
      if(FileIsEnding(file_handle_old_hst)) break;
      double   old_hst_close   = FileReadDouble (file_handle_old_hst);
      if(FileIsEnding(file_handle_old_hst)) break;
      double   old_hst_volume  = FileReadDouble (file_handle_old_hst);
      if(FileIsEnding(file_handle_old_hst)) break;
      FileWriteInteger(file_handle_backup,old_hst_bartime);
      FileWriteDouble (file_handle_backup,old_hst_open);
      FileWriteDouble (file_handle_backup,old_hst_low);
      FileWriteDouble (file_handle_backup,old_hst_high);
	   FileWriteDouble (file_handle_backup,old_hst_close);
	   FileWriteDouble (file_handle_backup,old_hst_volume);
     }
   FileClose(file_handle_backup);
   FileClose(file_handle_old_hst);
   file_name = StringConcatenate(symbol,"1.hst");
   int file_handle_new_hst = FileOpenHistory(file_name,FILE_BIN|FILE_WRITE);
   if(file_handle_new_hst < 0)
     {
      Print(str_err_01,file_name,"\".");
      str_comment = StringConcatenate(str_comment,"ImportData - ",str_mes_03);
      Comment(str_comment);
	   return;
     }
   file_name = StringConcatenate(ScriptName,"_FindData_",StringSubstr(symbol,0,6),".csv");
   int file_handle_csv = FileOpen(file_name,FILE_READ|FILE_CSV);
   if(file_handle_csv < 0)
     {
      Print(str_err_01,file_name,"\".");
      FileClose(file_handle_new_hst);
      str_comment = StringConcatenate(str_comment,"ImportData - ",str_mes_03);
      Comment(str_comment);
      return;
     }
   file_name = StringConcatenate(ScriptName,"_old_",symbol,"1.hst");
   file_handle_old_hst = FileOpenHistory(file_name,FILE_BIN|FILE_READ);
   if(file_handle_old_hst < 0)
     {
      Print(str_err_01,file_name,"\".");
      FileClose(file_handle_new_hst);
      FileClose(file_handle_csv);
      str_comment = StringConcatenate(str_comment,"ImportData - ",str_mes_03);
      Comment(str_comment);
	   return;
     }
 	hst_version = FileReadInteger(file_handle_old_hst);
 	FileWriteInteger(file_handle_new_hst,hst_version);
 	hst_copyright = FileReadString(file_handle_old_hst,64);
  	FileWriteString (file_handle_new_hst,hst_copyright,64);
  	hst_symbol = FileReadString(file_handle_old_hst,12);
   FileWriteString (file_handle_new_hst,hst_symbol,12);
   hst_period = FileReadInteger(file_handle_old_hst);
  	FileWriteInteger(file_handle_new_hst,hst_period);
  	hst_digits = FileReadInteger(file_handle_old_hst);
   FileWriteInteger(file_handle_new_hst,hst_digits);
   hst_timesign = FileReadInteger(file_handle_old_hst);
  	FileWriteInteger(file_handle_new_hst,hst_timesign);
  	hst_last_sync = FileReadInteger(file_handle_old_hst);
   FileWriteInteger(file_handle_new_hst,hst_last_sync);
   for(i=0; i<13; i++)
     {
      hst_reserved = FileReadInteger(file_handle_old_hst);
      FileWriteInteger(file_handle_new_hst,hst_reserved);
     }
   FileFlush(file_handle_new_hst);
   bool have_old_hst_file = false;
   while(true)
     {
      if(FileIsEnding(file_handle_csv)) break;
      string csv_date   = FileReadString(file_handle_csv);
      if(FileIsLineEnding(file_handle_csv)) continue;
      string csv_time   = FileReadString(file_handle_csv);
      if(FileIsLineEnding(file_handle_csv)) continue;
      double csv_open   = StrToDouble(FileReadString(file_handle_csv));
      if(FileIsLineEnding(file_handle_csv)) continue;
      double csv_high   = StrToDouble(FileReadString(file_handle_csv));
      if(FileIsLineEnding(file_handle_csv)) continue;
      double csv_low    = StrToDouble(FileReadString(file_handle_csv));
      if(FileIsLineEnding(file_handle_csv)) continue;
      double csv_close  = StrToDouble(FileReadString(file_handle_csv));
      if(FileIsLineEnding(file_handle_csv)) continue;
      double csv_volume = StrToDouble(FileReadString(file_handle_csv));
      datetime csv_bartime = StrToTime(StringConcatenate(StringSubstr(csv_date,8,2),".",StringSubstr(csv_date,5,2),".",StringSubstr(csv_date,0,4)," ",csv_time));
      while(true)
        {
         if(!have_old_hst_file)
           {
            if(!FileIsEnding(file_handle_old_hst))
              {
               have_old_hst_file = true;
               old_hst_bartime = FileReadInteger(file_handle_old_hst);
               old_hst_open    = FileReadDouble (file_handle_old_hst);
               old_hst_low     = FileReadDouble (file_handle_old_hst);
               old_hst_high    = FileReadDouble (file_handle_old_hst);
               old_hst_close   = FileReadDouble (file_handle_old_hst);
               old_hst_volume  = FileReadDouble (file_handle_old_hst);
              }
           }
         if(have_old_hst_file && old_hst_bartime < csv_bartime)
           {
            FileWriteInteger(file_handle_new_hst,old_hst_bartime);
            FileWriteDouble (file_handle_new_hst,old_hst_open);
            FileWriteDouble (file_handle_new_hst,old_hst_low);
            FileWriteDouble (file_handle_new_hst,old_hst_high);
            FileWriteDouble (file_handle_new_hst,old_hst_close);
            FileWriteDouble (file_handle_new_hst,old_hst_volume);
            have_old_hst_file = false;
           }
         else
           {
            FileWriteInteger(file_handle_new_hst,csv_bartime);
            FileWriteDouble (file_handle_new_hst,csv_open);
            FileWriteDouble (file_handle_new_hst,csv_low);
            FileWriteDouble (file_handle_new_hst,csv_high);
            FileWriteDouble (file_handle_new_hst,csv_close);
            FileWriteDouble (file_handle_new_hst,csv_volume);
            if(have_old_hst_file) if(old_hst_bartime == csv_bartime) have_old_hst_file = false;
            break;
           }
        }
     }
   if(have_old_hst_file)
     {
      FileWriteInteger(file_handle_new_hst,old_hst_bartime);
      FileWriteDouble (file_handle_new_hst,old_hst_open);
      FileWriteDouble (file_handle_new_hst,old_hst_low);
      FileWriteDouble (file_handle_new_hst,old_hst_high);
	   FileWriteDouble (file_handle_new_hst,old_hst_close);
	   FileWriteDouble (file_handle_new_hst,old_hst_volume);
     }
   while(true)
     {
      if(FileIsEnding(file_handle_old_hst)) break;
      old_hst_bartime = FileReadInteger(file_handle_old_hst);
      old_hst_open    = FileReadDouble (file_handle_old_hst);
      old_hst_low     = FileReadDouble (file_handle_old_hst);
      old_hst_high    = FileReadDouble (file_handle_old_hst);
      old_hst_close   = FileReadDouble (file_handle_old_hst);
      old_hst_volume  = FileReadDouble (file_handle_old_hst);
      FileWriteInteger(file_handle_new_hst,old_hst_bartime);
      FileWriteDouble (file_handle_new_hst,old_hst_open);
      FileWriteDouble (file_handle_new_hst,old_hst_low);
      FileWriteDouble (file_handle_new_hst,old_hst_high);
	   FileWriteDouble (file_handle_new_hst,old_hst_close);
	   FileWriteDouble (file_handle_new_hst,old_hst_volume);
     }
   FileClose(file_handle_new_hst);
   FileClose(file_handle_old_hst);
   FileClose(file_handle_csv);
   str_comment = StringConcatenate(str_comment,"ImportData - ",str_mes_07);
   Comment(str_comment);
  }

//---------------------------------------------------------------------------------------------+
//---- PeriodConverterEx ----------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
void PeriodConverterEx()
  {
   str_comment = StringConcatenate(str_comment,"PeriodConverterEx:\n");
   Comment(str_comment);
   for(int k=2; k<10; k++)
     {
      int k_period = GetTimeFrameByCode(k);
      str_comment = StringConcatenate(str_comment,TimeFrameToStr(k_period),": ");
      string file_name = StringConcatenate(symbol,DoubleToStr(k_period,0),".hst");
      int file_handle_hst = FileOpenHistory(file_name,FILE_BIN|FILE_WRITE);
      if(file_handle_hst < 0)
        {
         Print(str_err_01,file_name,"\".");
         str_comment = StringConcatenate(str_comment,str_mes_03);
	      continue;
        }
    	Comment(StringConcatenate(str_comment,str_mes_04));
      FileWriteInteger(file_handle_hst,400);
      FileWriteString (file_handle_hst,"(C)opyright 2003, MetaQuotes Software Corp.",64);
      FileWriteString (file_handle_hst,symbol,12);
      FileWriteInteger(file_handle_hst,k_period);
      FileWriteInteger(file_handle_hst,Digits);
      FileWriteInteger(file_handle_hst,0);
      FileWriteInteger(file_handle_hst,0);
      for(int i=0; i<13; i++) FileWriteInteger(file_handle_hst,0);
      int k_num_bars = 0;
      int period_sec = k_period * 60;
      int start_pos = iBars(NULL,PERIOD_M1) - 1;
      double k_open   = iOpen  (NULL,PERIOD_M1,start_pos);
      double k_low    = iLow   (NULL,PERIOD_M1,start_pos);
      double k_high   = iHigh  (NULL,PERIOD_M1,start_pos);
      double k_close  = iClose (NULL,PERIOD_M1,start_pos);
      double k_volume = iVolume(NULL,PERIOD_M1,start_pos);
      int k_time = iTime(NULL,PERIOD_M1,start_pos) / period_sec;
      k_time *= period_sec;
      for(i=start_pos-1; i>=0; i--)
        {
         datetime i_time = iTime  (NULL,PERIOD_M1,i);
         double i_open   = iOpen  (NULL,PERIOD_M1,i);
         double i_low    = iLow   (NULL,PERIOD_M1,i);
         double i_high   = iHigh  (NULL,PERIOD_M1,i);
         double i_close  = iClose (NULL,PERIOD_M1,i);
         double i_volume = iVolume(NULL,PERIOD_M1,i);
         if(i == 0) if(RefreshRates()) i = iBarShift(NULL,PERIOD_M1,i_time);
         if(i_time >= (k_time + period_sec) || i == 0)
           {
            if(i==0 && i_time < (k_time + period_sec))
              {
               k_volume += i_volume;
               if(i_low  < k_low)  k_low  = i_low;
               if(i_high > k_high) k_high = i_high;
               k_close = i_close;
              }
            FileWriteInteger(file_handle_hst,k_time);
            FileWriteDouble (file_handle_hst,k_open);
            FileWriteDouble (file_handle_hst,k_low);
            FileWriteDouble (file_handle_hst,k_high);
            FileWriteDouble (file_handle_hst,k_close);
            FileWriteDouble (file_handle_hst,k_volume);
            FileFlush(file_handle_hst);
            k_num_bars++;
            if(i_time >= (k_time + period_sec))
              {
               k_time   = i_time / period_sec;
               k_time  *= period_sec;
               k_open   = i_open;
               k_low    = i_low;
               k_high   = i_high;
               k_close  = i_close;
               k_volume = i_volume;
              }
           }
         else
           {
            k_volume += i_volume;
            if(i_low  < k_low)  k_low  = i_low;
            if(i_high > k_high) k_high = i_high;
            k_close = i_close;
           }
        } 
      FileClose(file_handle_hst);
      Print(StringConcatenate(TimeFrameToStr(k_period)," - ",str_mes_05,DoubleToStr(k_num_bars,0),str_mes_06,"."));
      str_comment = StringConcatenate(str_comment,str_mes_05,DoubleToStr(k_num_bars,0),str_mes_06,"\n");
     }
 	Comment(str_comment);
 	Sleep(1000);
  }

//---------------------------------------------------------------------------------------------+
//---- GetTimeFrameByCode ---------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
int GetTimeFrameByCode(int code_of_timeframe)
  {
   static int Periods[10] = {0,1,5,15,30,60,240,1440,10080,43200};
   if(code_of_timeframe < 0 || code_of_timeframe > 9) code_of_timeframe = 0;
   return(Periods[code_of_timeframe]);
  }

//---------------------------------------------------------------------------------------------+
//---- TimeFrameToStr -------------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
string TimeFrameToStr(int period)
  {
   switch(period)
     {
      case     0: string str_timeframe = TimeFrameToStr(Period()); break;
      case     1:        str_timeframe = "M1";          break;
      case     5:        str_timeframe = "M5";          break;
      case    15:        str_timeframe = "M15";         break;
      case    30:        str_timeframe = "M30";         break;
      case    60:        str_timeframe = "H1";          break;
      case   240:        str_timeframe = "H4";          break;
      case  1440:        str_timeframe = "D1";          break;
      case 10080:        str_timeframe = "W1";          break;
      case 43200:        str_timeframe = "MN";          break;
      default:           str_timeframe = StringConcatenate("M",DoubleToStr(period,0));
     }
   return(str_timeframe);
  }

//---------------------------------------------------------------------------------------------+
//---- IntegerToStr ---------------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
string IntegerToStr(int integer_number, int minimum_chars, string add_char = "0")
  {
   string result = DoubleToStr(integer_number,0);
   for(int i=minimum_chars-StringLen(result); i>0; i--) result = StringConcatenate(add_char,result);
   return(result);
  }

//---------------------------------------------------------------------------------------------+
//---- DayOfWeekToStr -------------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
string DayOfWeekToStr(int day_of_week, bool short_string=false)
  {
   if(!short_string)
     {
      switch(day_of_week)
        {
         case 0: string result = "Sunday";    break;
         case 1:        result = "Monday";    break;
         case 2:        result = "Tuesday";   break;
         case 3:        result = "Wednesday"; break;
         case 4:        result = "Thursday";  break;
         case 5:        result = "Friday";    break;
         case 6:        result = "Saturday";  break;
         default:       result = "Unknown";   break;
        }
     }
   else
     {
      switch(day_of_week)
        {
         case 0:  result = "Sun"; break;
         case 1:  result = "Mon"; break;
         case 2:  result = "Tue"; break;
         case 3:  result = "Wed"; break;
         case 4:  result = "Thu"; break;
         case 5:  result = "Fri"; break;
         case 6:  result = "Sat"; break;
         default: result = "Unk"; break;
        }
     }
   return(result);
  }

//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+